@@ -7,6 +7,9 @@ else:
     import unittest  # pragma: nocover
 
 from webtest import TestApp
+import six
+from six import b as b_
+from six.moves import cStringIO as StringIO
 
 from pecan import (
     Pecan, expose, request, response, redirect, abort, make_app,
@@ -44,17 +47,17 @@ class TestIndexRouting(PecanTestCase):
     def test_empty_root(self):
         r = self.app_.get('/')
         assert r.status_int == 200
-        assert r.body == 'Hello, World!'
+        assert r.body == b_('Hello, World!')
 
     def test_index(self):
         r = self.app_.get('/index')
         assert r.status_int == 200
-        assert r.body == 'Hello, World!'
+        assert r.body == b_('Hello, World!')
 
     def test_index_html(self):
         r = self.app_.get('/index.html')
         assert r.status_int == 200
-        assert r.body == 'Hello, World!'
+        assert r.body == b_('Hello, World!')
 
 
 class TestObjectDispatch(PecanTestCase):
@@ -97,22 +100,22 @@ class TestObjectDispatch(PecanTestCase):
     def test_index(self):
         r = self.app_.get('/')
         assert r.status_int == 200
-        assert r.body == '/'
+        assert r.body == b_('/')
 
     def test_one_level(self):
         r = self.app_.get('/deeper')
         assert r.status_int == 200
-        assert r.body == '/deeper'
+        assert r.body == b_('/deeper')
 
     def test_one_level_with_trailing(self):
         r = self.app_.get('/sub/')
         assert r.status_int == 200
-        assert r.body == '/sub/'
+        assert r.body == b_('/sub/')
 
     def test_two_levels(self):
         r = self.app_.get('/sub/deeper')
         assert r.status_int == 200
-        assert r.body == '/sub/deeper'
+        assert r.body == b_('/sub/deeper')
 
     def test_two_levels_with_trailing(self):
         r = self.app_.get('/sub/sub/')
@@ -121,7 +124,7 @@ class TestObjectDispatch(PecanTestCase):
     def test_three_levels(self):
         r = self.app_.get('/sub/sub/deeper')
         assert r.status_int == 200
-        assert r.body == '/sub/sub/deeper'
+        assert r.body == b_('/sub/sub/deeper')
 
 
 class TestLookups(PecanTestCase):
@@ -154,17 +157,17 @@ class TestLookups(PecanTestCase):
     def test_index(self):
         r = self.app_.get('/')
         assert r.status_int == 200
-        assert r.body == '/'
+        assert r.body == b_('/')
 
     def test_lookup(self):
         r = self.app_.get('/100/')
         assert r.status_int == 200
-        assert r.body == '/100'
+        assert r.body == b_('/100')
 
     def test_lookup_with_method(self):
         r = self.app_.get('/100/name')
         assert r.status_int == 200
-        assert r.body == '/100/name'
+        assert r.body == b_('/100/name')
 
     def test_lookup_with_wrong_argspec(self):
         class RootController(object):
@@ -179,6 +182,35 @@ class TestLookups(PecanTestCase):
             assert r.status_int == 404
 
 
+class TestCanonicalLookups(PecanTestCase):
+
+    @property
+    def app_(self):
+        class LookupController(object):
+            def __init__(self, someID):
+                self.someID = someID
+
+            @expose()
+            def index(self):
+                return self.someID
+
+        class UserController(object):
+            @expose()
+            def _lookup(self, someID, *remainder):
+                return LookupController(someID), remainder
+
+        class RootController(object):
+            users = UserController()
+
+        return TestApp(Pecan(RootController()))
+
+    def test_canonical_lookup(self):
+        assert self.app_.get('/users', expect_errors=404).status_int == 404
+        assert self.app_.get('/users/', expect_errors=404).status_int == 404
+        assert self.app_.get('/users/100').status_int == 302
+        assert self.app_.get('/users/100/').body == b_('100')
+
+
 class TestControllerArguments(PecanTestCase):
 
     @property
@@ -245,19 +277,22 @@ class TestControllerArguments(PecanTestCase):
         try:
             r = self.app_.get('/')
             assert r.status_int != 200  # pragma: nocover
-        except Exception, ex:
+        except Exception as ex:
             assert type(ex) == TypeError
-            assert ex.args[0] == 'index() takes exactly 2 arguments (1 given)'
+            assert ex.args[0] in (
+                "index() takes exactly 2 arguments (1 given)",
+                "index() missing 1 required positional argument: 'id'"
+            )  # this messaging changed in Python 3.3
 
     def test_single_argument(self):
         r = self.app_.get('/1')
         assert r.status_int == 200
-        assert r.body == 'index: 1'
+        assert r.body == b_('index: 1')
 
     def test_single_argument_with_encoded_url(self):
         r = self.app_.get('/This%20is%20a%20test%21')
         assert r.status_int == 200
-        assert r.body == 'index: This is a test!'
+        assert r.body == b_('index: This is a test!')
 
     def test_two_arguments(self):
         r = self.app_.get('/1/dummy', status=404)
@@ -266,90 +301,90 @@ class TestControllerArguments(PecanTestCase):
     def test_keyword_argument(self):
         r = self.app_.get('/?id=2')
         assert r.status_int == 200
-        assert r.body == 'index: 2'
+        assert r.body == b_('index: 2')
 
     def test_keyword_argument_with_encoded_url(self):
         r = self.app_.get('/?id=This%20is%20a%20test%21')
         assert r.status_int == 200
-        assert r.body == 'index: This is a test!'
+        assert r.body == b_('index: This is a test!')
 
     def test_argument_and_keyword_argument(self):
         r = self.app_.get('/3?id=three')
         assert r.status_int == 200
-        assert r.body == 'index: 3'
+        assert r.body == b_('index: 3')
 
     def test_encoded_argument_and_keyword_argument(self):
         r = self.app_.get('/This%20is%20a%20test%21?id=three')
         assert r.status_int == 200
-        assert r.body == 'index: This is a test!'
+        assert r.body == b_('index: This is a test!')
 
     def test_explicit_kwargs(self):
         r = self.app_.post('/', {'id': '4'})
         assert r.status_int == 200
-        assert r.body == 'index: 4'
+        assert r.body == b_('index: 4')
 
     def test_path_with_explicit_kwargs(self):
         r = self.app_.post('/4', {'id': 'four'})
         assert r.status_int == 200
-        assert r.body == 'index: 4'
+        assert r.body == b_('index: 4')
 
     def test_multiple_kwargs(self):
         r = self.app_.get('/?id=5&dummy=dummy')
         assert r.status_int == 200
-        assert r.body == 'index: 5'
+        assert r.body == b_('index: 5')
 
     def test_kwargs_from_root(self):
         r = self.app_.post('/', {'id': '6', 'dummy': 'dummy'})
         assert r.status_int == 200
-        assert r.body == 'index: 6'
+        assert r.body == b_('index: 6')
 
         # multiple args
 
     def test_multiple_positional_arguments(self):
         r = self.app_.get('/multiple/one/two')
         assert r.status_int == 200
-        assert r.body == 'multiple: one, two'
+        assert r.body == b_('multiple: one, two')
 
     def test_multiple_positional_arguments_with_url_encode(self):
         r = self.app_.get('/multiple/One%20/Two%21')
         assert r.status_int == 200
-        assert r.body == 'multiple: One , Two!'
+        assert r.body == b_('multiple: One , Two!')
 
     def test_multiple_positional_arguments_with_kwargs(self):
         r = self.app_.get('/multiple?one=three&two=four')
         assert r.status_int == 200
-        assert r.body == 'multiple: three, four'
+        assert r.body == b_('multiple: three, four')
 
     def test_multiple_positional_arguments_with_url_encoded_kwargs(self):
         r = self.app_.get('/multiple?one=Three%20&two=Four%20%21')
         assert r.status_int == 200
-        assert r.body == 'multiple: Three , Four !'
+        assert r.body == b_('multiple: Three , Four !')
 
     def test_positional_args_with_dictionary_kwargs(self):
         r = self.app_.post('/multiple', {'one': 'five', 'two': 'six'})
         assert r.status_int == 200
-        assert r.body == 'multiple: five, six'
+        assert r.body == b_('multiple: five, six')
 
     def test_positional_args_with_url_encoded_dictionary_kwargs(self):
         r = self.app_.post('/multiple', {'one': 'Five%20', 'two': 'Six%20%21'})
         assert r.status_int == 200
-        assert r.body == 'multiple: Five%20, Six%20%21'
+        assert r.body == b_('multiple: Five%20, Six%20%21')
 
         # optional arg
     def test_optional_arg(self):
         r = self.app_.get('/optional')
         assert r.status_int == 200
-        assert r.body == 'optional: None'
+        assert r.body == b_('optional: None')
 
     def test_multiple_optional(self):
         r = self.app_.get('/optional/1')
         assert r.status_int == 200
-        assert r.body == 'optional: 1'
+        assert r.body == b_('optional: 1')
 
     def test_multiple_optional_url_encoded(self):
         r = self.app_.get('/optional/Some%20Number')
         assert r.status_int == 200
-        assert r.body == 'optional: Some Number'
+        assert r.body == b_('optional: Some Number')
 
     def test_multiple_optional_missing(self):
         r = self.app_.get('/optional/2/dummy', status=404)
@@ -358,57 +393,57 @@ class TestControllerArguments(PecanTestCase):
     def test_multiple_with_kwargs(self):
         r = self.app_.get('/optional?id=2')
         assert r.status_int == 200
-        assert r.body == 'optional: 2'
+        assert r.body == b_('optional: 2')
 
     def test_multiple_with_url_encoded_kwargs(self):
         r = self.app_.get('/optional?id=Some%20Number')
         assert r.status_int == 200
-        assert r.body == 'optional: Some Number'
+        assert r.body == b_('optional: Some Number')
 
     def test_multiple_args_with_url_encoded_kwargs(self):
         r = self.app_.get('/optional/3?id=three')
         assert r.status_int == 200
-        assert r.body == 'optional: 3'
+        assert r.body == b_('optional: 3')
 
     def test_url_encoded_positional_args(self):
         r = self.app_.get('/optional/Some%20Number?id=three')
         assert r.status_int == 200
-        assert r.body == 'optional: Some Number'
+        assert r.body == b_('optional: Some Number')
 
     def test_optional_arg_with_kwargs(self):
         r = self.app_.post('/optional', {'id': '4'})
         assert r.status_int == 200
-        assert r.body == 'optional: 4'
+        assert r.body == b_('optional: 4')
 
     def test_optional_arg_with_url_encoded_kwargs(self):
         r = self.app_.post('/optional', {'id': 'Some%20Number'})
         assert r.status_int == 200
-        assert r.body == 'optional: Some%20Number'
+        assert r.body == b_('optional: Some%20Number')
 
     def test_multiple_positional_arguments_with_dictionary_kwargs(self):
         r = self.app_.post('/optional/5', {'id': 'five'})
         assert r.status_int == 200
-        assert r.body == 'optional: 5'
+        assert r.body == b_('optional: 5')
 
     def test_multiple_positional_url_encoded_arguments_with_kwargs(self):
         r = self.app_.post('/optional/Some%20Number', {'id': 'five'})
         assert r.status_int == 200
-        assert r.body == 'optional: Some Number'
+        assert r.body == b_('optional: Some Number')
 
     def test_optional_arg_with_multiple_kwargs(self):
         r = self.app_.get('/optional?id=6&dummy=dummy')
         assert r.status_int == 200
-        assert r.body == 'optional: 6'
+        assert r.body == b_('optional: 6')
 
     def test_optional_arg_with_multiple_url_encoded_kwargs(self):
         r = self.app_.get('/optional?id=Some%20Number&dummy=dummy')
         assert r.status_int == 200
-        assert r.body == 'optional: Some Number'
+        assert r.body == b_('optional: Some Number')
 
     def test_optional_arg_with_multiple_dictionary_kwargs(self):
         r = self.app_.post('/optional', {'id': '7', 'dummy': 'dummy'})
         assert r.status_int == 200
-        assert r.body == 'optional: 7'
+        assert r.body == b_('optional: 7')
 
     def test_optional_arg_with_multiple_url_encoded_dictionary_kwargs(self):
         r = self.app_.post('/optional', {
@@ -416,34 +451,34 @@ class TestControllerArguments(PecanTestCase):
             'dummy': 'dummy'
         })
         assert r.status_int == 200
-        assert r.body == 'optional: Some%20Number'
+        assert r.body == b_('optional: Some%20Number')
 
         # multiple optional args
 
     def test_multiple_optional_positional_args(self):
         r = self.app_.get('/multiple_optional')
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: None, None, None'
+        assert r.body == b_('multiple_optional: None, None, None')
 
     def test_multiple_optional_positional_args_one_arg(self):
         r = self.app_.get('/multiple_optional/1')
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: 1, None, None'
+        assert r.body == b_('multiple_optional: 1, None, None')
 
     def test_multiple_optional_positional_args_one_url_encoded_arg(self):
         r = self.app_.get('/multiple_optional/One%21')
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: One!, None, None'
+        assert r.body == b_('multiple_optional: One!, None, None')
 
     def test_multiple_optional_positional_args_all_args(self):
         r = self.app_.get('/multiple_optional/1/2/3')
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: 1, 2, 3'
+        assert r.body == b_('multiple_optional: 1, 2, 3')
 
     def test_multiple_optional_positional_args_all_url_encoded_args(self):
         r = self.app_.get('/multiple_optional/One%21/Two%21/Three%21')
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: One!, Two!, Three!'
+        assert r.body == b_('multiple_optional: One!, Two!, Three!')
 
     def test_multiple_optional_positional_args_too_many_args(self):
         r = self.app_.get('/multiple_optional/1/2/3/dummy', status=404)
@@ -452,54 +487,54 @@ class TestControllerArguments(PecanTestCase):
     def test_multiple_optional_positional_args_with_kwargs(self):
         r = self.app_.get('/multiple_optional?one=1')
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: 1, None, None'
+        assert r.body == b_('multiple_optional: 1, None, None')
 
     def test_multiple_optional_positional_args_with_url_encoded_kwargs(self):
         r = self.app_.get('/multiple_optional?one=One%21')
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: One!, None, None'
+        assert r.body == b_('multiple_optional: One!, None, None')
 
     def test_multiple_optional_positional_args_with_string_kwargs(self):
         r = self.app_.get('/multiple_optional/1?one=one')
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: 1, None, None'
+        assert r.body == b_('multiple_optional: 1, None, None')
 
     def test_multiple_optional_positional_args_with_encoded_str_kwargs(self):
         r = self.app_.get('/multiple_optional/One%21?one=one')
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: One!, None, None'
+        assert r.body == b_('multiple_optional: One!, None, None')
 
     def test_multiple_optional_positional_args_with_dict_kwargs(self):
         r = self.app_.post('/multiple_optional', {'one': '1'})
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: 1, None, None'
+        assert r.body == b_('multiple_optional: 1, None, None')
 
     def test_multiple_optional_positional_args_with_encoded_dict_kwargs(self):
         r = self.app_.post('/multiple_optional', {'one': 'One%21'})
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: One%21, None, None'
+        assert r.body == b_('multiple_optional: One%21, None, None')
 
     def test_multiple_optional_positional_args_and_dict_kwargs(self):
         r = self.app_.post('/multiple_optional/1', {'one': 'one'})
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: 1, None, None'
+        assert r.body == b_('multiple_optional: 1, None, None')
 
     def test_multiple_optional_encoded_positional_args_and_dict_kwargs(self):
         r = self.app_.post('/multiple_optional/One%21', {'one': 'one'})
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: One!, None, None'
+        assert r.body == b_('multiple_optional: One!, None, None')
 
     def test_multiple_optional_args_with_multiple_kwargs(self):
         r = self.app_.get('/multiple_optional?one=1&two=2&three=3&four=4')
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: 1, 2, 3'
+        assert r.body == b_('multiple_optional: 1, 2, 3')
 
     def test_multiple_optional_args_with_multiple_encoded_kwargs(self):
         r = self.app_.get(
             '/multiple_optional?one=One%21&two=Two%21&three=Three%21&four=4'
         )
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: One!, Two!, Three!'
+        assert r.body == b_('multiple_optional: One!, Two!, Three!')
 
     def test_multiple_optional_args_with_multiple_dict_kwargs(self):
         r = self.app_.post(
@@ -507,7 +542,7 @@ class TestControllerArguments(PecanTestCase):
             {'one': '1', 'two': '2', 'three': '3', 'four': '4'}
         )
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: 1, 2, 3'
+        assert r.body == b_('multiple_optional: 1, 2, 3')
 
     def test_multiple_optional_args_with_multiple_encoded_dict_kwargs(self):
         r = self.app_.post(
@@ -520,52 +555,52 @@ class TestControllerArguments(PecanTestCase):
             }
         )
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: One%21, Two%21, Three%21'
+        assert r.body == b_('multiple_optional: One%21, Two%21, Three%21')
 
     def test_multiple_optional_args_with_last_kwarg(self):
         r = self.app_.get('/multiple_optional?three=3')
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: None, None, 3'
+        assert r.body == b_('multiple_optional: None, None, 3')
 
     def test_multiple_optional_args_with_last_encoded_kwarg(self):
         r = self.app_.get('/multiple_optional?three=Three%21')
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: None, None, Three!'
+        assert r.body == b_('multiple_optional: None, None, Three!')
 
     def test_multiple_optional_args_with_middle_arg(self):
         r = self.app_.get('/multiple_optional', {'two': '2'})
         assert r.status_int == 200
-        assert r.body == 'multiple_optional: None, 2, None'
+        assert r.body == b_('multiple_optional: None, 2, None')
 
     def test_variable_args(self):
         r = self.app_.get('/variable_args')
         assert r.status_int == 200
-        assert r.body == 'variable_args: '
+        assert r.body == b_('variable_args: ')
 
     def test_multiple_variable_args(self):
         r = self.app_.get('/variable_args/1/dummy')
         assert r.status_int == 200
-        assert r.body == 'variable_args: 1, dummy'
+        assert r.body == b_('variable_args: 1, dummy')
 
     def test_multiple_encoded_variable_args(self):
         r = self.app_.get('/variable_args/Testing%20One%20Two/Three%21')
         assert r.status_int == 200
-        assert r.body == 'variable_args: Testing One Two, Three!'
+        assert r.body == b_('variable_args: Testing One Two, Three!')
 
     def test_variable_args_with_kwargs(self):
         r = self.app_.get('/variable_args?id=2&dummy=dummy')
         assert r.status_int == 200
-        assert r.body == 'variable_args: '
+        assert r.body == b_('variable_args: ')
 
     def test_variable_args_with_dict_kwargs(self):
         r = self.app_.post('/variable_args', {'id': '3', 'dummy': 'dummy'})
         assert r.status_int == 200
-        assert r.body == 'variable_args: '
+        assert r.body == b_('variable_args: ')
 
     def test_variable_kwargs(self):
         r = self.app_.get('/variable_kwargs')
         assert r.status_int == 200
-        assert r.body == 'variable_kwargs: '
+        assert r.body == b_('variable_kwargs: ')
 
     def test_multiple_variable_kwargs(self):
         r = self.app_.get('/variable_kwargs/1/dummy', status=404)
@@ -574,19 +609,19 @@ class TestControllerArguments(PecanTestCase):
     def test_multiple_variable_kwargs_with_explicit_kwargs(self):
         r = self.app_.get('/variable_kwargs?id=2&dummy=dummy')
         assert r.status_int == 200
-        assert r.body == 'variable_kwargs: dummy=dummy, id=2'
+        assert r.body == b_('variable_kwargs: dummy=dummy, id=2')
 
     def test_multiple_variable_kwargs_with_explicit_encoded_kwargs(self):
         r = self.app_.get(
             '/variable_kwargs?id=Two%21&dummy=This%20is%20a%20test'
         )
         assert r.status_int == 200
-        assert r.body == 'variable_kwargs: dummy=This is a test, id=Two!'
+        assert r.body == b_('variable_kwargs: dummy=This is a test, id=Two!')
 
     def test_multiple_variable_kwargs_with_dict_kwargs(self):
         r = self.app_.post('/variable_kwargs', {'id': '3', 'dummy': 'dummy'})
         assert r.status_int == 200
-        assert r.body == 'variable_kwargs: dummy=dummy, id=3'
+        assert r.body == b_('variable_kwargs: dummy=dummy, id=3')
 
     def test_multiple_variable_kwargs_with_encoded_dict_kwargs(self):
         r = self.app_.post(
@@ -595,42 +630,42 @@ class TestControllerArguments(PecanTestCase):
         )
         assert r.status_int == 200
         result = 'variable_kwargs: dummy=This%20is%20a%20test, id=Three%21'
-        assert r.body == result
+        assert r.body == b_(result)
 
     def test_variable_all(self):
         r = self.app_.get('/variable_all')
         assert r.status_int == 200
-        assert r.body == 'variable_all: '
+        assert r.body == b_('variable_all: ')
 
     def test_variable_all_with_one_extra(self):
         r = self.app_.get('/variable_all/1')
         assert r.status_int == 200
-        assert r.body == 'variable_all: 1'
+        assert r.body == b_('variable_all: 1')
 
     def test_variable_all_with_two_extras(self):
         r = self.app_.get('/variable_all/2/dummy')
         assert r.status_int == 200
-        assert r.body == 'variable_all: 2, dummy'
+        assert r.body == b_('variable_all: 2, dummy')
 
     def test_variable_mixed(self):
         r = self.app_.get('/variable_all/3?month=1&day=12')
         assert r.status_int == 200
-        assert r.body == 'variable_all: 3, day=12, month=1'
+        assert r.body == b_('variable_all: 3, day=12, month=1')
 
     def test_variable_mixed_explicit(self):
         r = self.app_.get('/variable_all/4?id=four&month=1&day=12')
         assert r.status_int == 200
-        assert r.body == 'variable_all: 4, day=12, id=four, month=1'
+        assert r.body == b_('variable_all: 4, day=12, id=four, month=1')
 
     def test_variable_post(self):
         r = self.app_.post('/variable_all/5/dummy')
         assert r.status_int == 200
-        assert r.body == 'variable_all: 5, dummy'
+        assert r.body == b_('variable_all: 5, dummy')
 
     def test_variable_post_with_kwargs(self):
         r = self.app_.post('/variable_all/6', {'month': '1', 'day': '12'})
         assert r.status_int == 200
-        assert r.body == 'variable_all: 6, day=12, month=1'
+        assert r.body == b_('variable_all: 6, day=12, month=1')
 
     def test_variable_post_mixed(self):
         r = self.app_.post(
@@ -638,60 +673,63 @@ class TestControllerArguments(PecanTestCase):
             {'id': 'seven', 'month': '1', 'day': '12'}
         )
         assert r.status_int == 200
-        assert r.body == 'variable_all: 7, day=12, id=seven, month=1'
+        assert r.body == b_('variable_all: 7, day=12, id=seven, month=1')
 
     def test_no_remainder(self):
         try:
             r = self.app_.get('/eater')
             assert r.status_int != 200  # pragma: nocover
-        except Exception, ex:
+        except Exception as ex:
             assert type(ex) == TypeError
-            assert ex.args[0] == 'eater() takes at least 2 arguments (1 given)'
+            assert ex.args[0] in (
+                "eater() takes at least 2 arguments (1 given)",
+                "eater() missing 1 required positional argument: 'id'"
+            )  # this messaging changed in Python 3.3
 
     def test_one_remainder(self):
         r = self.app_.get('/eater/1')
         assert r.status_int == 200
-        assert r.body == 'eater: 1, None, '
+        assert r.body == b_('eater: 1, None, ')
 
     def test_two_remainders(self):
         r = self.app_.get('/eater/2/dummy')
         assert r.status_int == 200
-        assert r.body == 'eater: 2, dummy, '
+        assert r.body == b_('eater: 2, dummy, ')
 
     def test_many_remainders(self):
         r = self.app_.get('/eater/3/dummy/foo/bar')
         assert r.status_int == 200
-        assert r.body == 'eater: 3, dummy, foo, bar'
+        assert r.body == b_('eater: 3, dummy, foo, bar')
 
     def test_remainder_with_kwargs(self):
         r = self.app_.get('/eater/4?month=1&day=12')
         assert r.status_int == 200
-        assert r.body == 'eater: 4, None, day=12, month=1'
+        assert r.body == b_('eater: 4, None, day=12, month=1')
 
     def test_remainder_with_many_kwargs(self):
         r = self.app_.get('/eater/5?id=five&month=1&day=12&dummy=dummy')
         assert r.status_int == 200
-        assert r.body == 'eater: 5, dummy, day=12, month=1'
+        assert r.body == b_('eater: 5, dummy, day=12, month=1')
 
     def test_post_remainder(self):
         r = self.app_.post('/eater/6')
         assert r.status_int == 200
-        assert r.body == 'eater: 6, None, '
+        assert r.body == b_('eater: 6, None, ')
 
     def test_post_three_remainders(self):
         r = self.app_.post('/eater/7/dummy')
         assert r.status_int == 200
-        assert r.body == 'eater: 7, dummy, '
+        assert r.body == b_('eater: 7, dummy, ')
 
     def test_post_many_remainders(self):
         r = self.app_.post('/eater/8/dummy/foo/bar')
         assert r.status_int == 200
-        assert r.body == 'eater: 8, dummy, foo, bar'
+        assert r.body == b_('eater: 8, dummy, foo, bar')
 
     def test_post_remainder_with_kwargs(self):
         r = self.app_.post('/eater/9', {'month': '1', 'day': '12'})
         assert r.status_int == 200
-        assert r.body == 'eater: 9, None, day=12, month=1'
+        assert r.body == b_('eater: 9, None, day=12, month=1')
 
     def test_post_many_remainders_with_many_kwargs(self):
         r = self.app_.post(
@@ -699,7 +737,7 @@ class TestControllerArguments(PecanTestCase):
             {'id': 'ten', 'month': '1', 'day': '12', 'dummy': 'dummy'}
         )
         assert r.status_int == 200
-        assert r.body == 'eater: 10, dummy, day=12, month=1'
+        assert r.body == b_('eater: 10, dummy, day=12, month=1')
 
 
 class TestAbort(PecanTestCase):
@@ -774,12 +812,12 @@ class TestRedirect(PecanTestCase):
         assert r.status_int == 302
         r = r.follow()
         assert r.status_int == 200
-        assert r.body == 'it worked!'
+        assert r.body == b_('it worked!')
 
     def test_internal(self):
         r = self.app_.get('/internal')
         assert r.status_int == 200
-        assert r.body == 'it worked!'
+        assert r.body == b_('it worked!')
 
     def test_internal_with_301(self):
         self.assertRaises(ValueError, self.app_.get, '/bad_internal')
@@ -789,7 +827,7 @@ class TestRedirect(PecanTestCase):
         assert r.status_int == 301
         r = r.follow()
         assert r.status_int == 200
-        assert r.body == 'it worked!'
+        assert r.body == b_('it worked!')
 
     def test_x_forward_proto(self):
         class ChildController(object):
@@ -821,14 +859,13 @@ class TestRedirect(PecanTestCase):
 class TestStreamedResponse(PecanTestCase):
 
     def test_streaming_response(self):
-        import StringIO
 
         class RootController(object):
             @expose(content_type='text/plain')
             def test(self, foo):
                 if foo == 'stream':
                     # mimic large file
-                    contents = StringIO.StringIO('stream')
+                    contents = six.BytesIO(b_('stream'))
                     response.content_type = 'application/octet-stream'
                     contents.seek(0, os.SEEK_END)
                     response.content_length = contents.tell()
@@ -841,11 +878,11 @@ class TestStreamedResponse(PecanTestCase):
         app = TestApp(Pecan(RootController()))
         r = app.get('/test/stream')
         assert r.content_type == 'application/octet-stream'
-        assert r.body == 'stream'
+        assert r.body == b_('stream')
 
         r = app.get('/test/plain')
         assert r.content_type == 'text/plain'
-        assert r.body == 'plain text'
+        assert r.body == b_('plain text')
 
 
 class TestThreadLocalState(PecanTestCase):
@@ -865,7 +902,7 @@ class TestThreadLocalState(PecanTestCase):
         app = TestApp(Pecan(RootController()))
         r = app.get('/')
         assert r.status_int == 200
-        assert r.body == '/'
+        assert r.body == b_('/')
 
     def test_request_state_cleanup(self):
         """
@@ -882,9 +919,9 @@ class TestThreadLocalState(PecanTestCase):
         app = TestApp(Pecan(RootController()))
         r = app.get('/')
         assert r.status_int == 200
-        assert r.body == '/'
+        assert r.body == b_('/')
 
-        assert state.__dict__.keys() == ['app']
+        assert list(state.__dict__.keys()) == ['app']
 
 
 class TestFileTypeExtensions(PecanTestCase):
@@ -908,22 +945,22 @@ class TestFileTypeExtensions(PecanTestCase):
     def test_html_extension(self):
         r = self.app_.get('/index.html')
         assert r.status_int == 200
-        assert r.body == '.html'
+        assert r.body == b_('.html')
 
     def test_image_extension(self):
         r = self.app_.get('/image.png')
         assert r.status_int == 200
-        assert r.body == '.png'
+        assert r.body == b_('.png')
 
     def test_hidden_file(self):
         r = self.app_.get('/.vimrc')
         assert r.status_int == 200
-        assert r.body == ''
+        assert r.body == b_('')
 
     def test_multi_dot_extension(self):
         r = self.app_.get('/gradient.min.js')
         assert r.status_int == 200
-        assert r.body == '.js'
+        assert r.body == b_('.js')
 
     def test_bad_content_type(self):
         class RootController(object):
@@ -934,11 +971,11 @@ class TestFileTypeExtensions(PecanTestCase):
         app = TestApp(Pecan(RootController()))
         r = app.get('/')
         assert r.status_int == 200
-        assert r.body == '/'
+        assert r.body == b_('/')
 
         r = app.get('/index.html', expect_errors=True)
         assert r.status_int == 200
-        assert r.body == '/'
+        assert r.body == b_('/')
 
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
@@ -957,7 +994,7 @@ class TestFileTypeExtensions(PecanTestCase):
 
         r = app.get('/example:x.tiny')
         assert r.status_int == 200
-        assert r.body == 'SOME VALUE'
+        assert r.body == b_('SOME VALUE')
 
     def test_guessing_disabled(self):
         class RootController(object):
@@ -972,7 +1009,7 @@ class TestFileTypeExtensions(PecanTestCase):
 
         r = app.get('/index.html')
         assert r.status_int == 200
-        assert r.body == 'SOME VALUE'
+        assert r.body == b_('SOME VALUE')
 
 
 class TestContentTypeByAcceptHeaders(PecanTestCase):
@@ -1057,12 +1094,12 @@ class TestCanonicalRouting(PecanTestCase):
     def test_root(self):
         r = self.app_.get('/')
         assert r.status_int == 200
-        assert 'index' in r.body
+        assert b_('index') in r.body
 
     def test_index(self):
         r = self.app_.get('/index')
         assert r.status_int == 200
-        assert 'index' in r.body
+        assert b_('index') in r.body
 
     def test_broken_clients(self):
         # for broken clients
@@ -1073,7 +1110,7 @@ class TestCanonicalRouting(PecanTestCase):
     def test_sub_controller_with_trailing(self):
         r = self.app_.get('/sub/')
         assert r.status_int == 200
-        assert 'subindex' in r.body
+        assert b_('subindex') in r.body
 
     def test_sub_controller_redirect(self):
         r = self.app_.get('/sub', status=302)
@@ -1090,23 +1127,23 @@ class TestCanonicalRouting(PecanTestCase):
         try:
             self.app_.post('/sub', dict(foo=1))
             raise Exception("Post should fail")  # pragma: nocover
-        except Exception, e:
+        except Exception as e:
             assert isinstance(e, RuntimeError)
 
     def test_with_args(self):
         r = self.app_.get('/arg/index/foo')
         assert r.status_int == 200
-        assert r.body == 'foo'
+        assert r.body == b_('foo')
 
     def test_accept_noncanonical(self):
         r = self.app_.get('/accept/')
         assert r.status_int == 200
-        assert 'accept' == r.body
+        assert r.body == b_('accept')
 
     def test_accept_noncanonical_no_trailing_slash(self):
         r = self.app_.get('/accept')
         assert r.status_int == 200
-        assert 'accept' == r.body
+        assert r.body == b_('accept')
 
 
 class TestNonCanonical(PecanTestCase):
@@ -1143,22 +1180,22 @@ class TestNonCanonical(PecanTestCase):
     def test_index(self):
         r = self.app_.get('/')
         assert r.status_int == 200
-        assert 'index' in r.body
+        assert b_('index') in r.body
 
     def test_subcontroller(self):
         r = self.app_.get('/sub')
         assert r.status_int == 200
-        assert 'subindex' in r.body
+        assert b_('subindex') in r.body
 
     def test_subcontroller_with_kwargs(self):
         r = self.app_.post('/sub', dict(foo=1))
         assert r.status_int == 200
-        assert 'subindex' in r.body
+        assert b_('subindex') in r.body
 
     def test_sub_controller_with_trailing(self):
         r = self.app_.get('/sub/')
         assert r.status_int == 200
-        assert 'subindex' in r.body
+        assert b_('subindex') in r.body
 
     def test_proxy(self):
         class RootController(object):
@@ -1198,7 +1235,6 @@ class TestLogging(PecanTestCase):
                 logging.getLogger('pecantesting').info('HELLO WORLD')
                 return "HELLO WORLD"
 
-        from cStringIO import StringIO
         f = StringIO()
 
         app = TestApp(make_app(RootController(), logging={
@@ -1227,7 +1263,6 @@ class TestLogging(PecanTestCase):
                 logging.getLogger('pecantesting').info('HELLO WORLD')
                 return "HELLO WORLD"
 
-        from cStringIO import StringIO
         f = StringIO()
 
         from pecan.configuration import conf_from_dict
@@ -1271,16 +1306,16 @@ class TestEngines(PecanTestCase):
         )
         r = app.get('/')
         assert r.status_int == 200
-        assert "<h1>Hello, Jonathan!</h1>" in r.body
+        assert b_("<h1>Hello, Jonathan!</h1>") in r.body
 
         r = app.get('/index.html?name=World')
         assert r.status_int == 200
-        assert "<h1>Hello, World!</h1>" in r.body
+        assert b_("<h1>Hello, World!</h1>") in r.body
 
         error_msg = None
         try:
             r = app.get('/badtemplate.html')
-        except Exception, e:
+        except Exception as e:
             for error_f in error_formatters:
                 error_msg = error_f(e)
                 if error_msg:
@@ -1300,11 +1335,11 @@ class TestEngines(PecanTestCase):
         )
         r = app.get('/')
         assert r.status_int == 200
-        assert "<h1>Hello, Jonathan!</h1>" in r.body
+        assert b_("<h1>Hello, Jonathan!</h1>") in r.body
 
         r = app.get('/index.html?name=World')
         assert r.status_int == 200
-        assert "<h1>Hello, World!</h1>" in r.body
+        assert b_("<h1>Hello, World!</h1>") in r.body
 
     @unittest.skipIf('jinja' not in builtin_renderers, 'Jinja not installed')
     def test_jinja(self):
@@ -1323,12 +1358,12 @@ class TestEngines(PecanTestCase):
         )
         r = app.get('/')
         assert r.status_int == 200
-        assert "<h1>Hello, Jonathan!</h1>" in r.body
+        assert b_("<h1>Hello, Jonathan!</h1>") in r.body
 
         error_msg = None
         try:
             r = app.get('/badtemplate.html')
-        except Exception, e:
+        except Exception as e:
             for error_f in error_formatters:
                 error_msg = error_f(e)
                 if error_msg:
@@ -1352,16 +1387,16 @@ class TestEngines(PecanTestCase):
         )
         r = app.get('/')
         assert r.status_int == 200
-        assert "<h1>Hello, Jonathan!</h1>" in r.body
+        assert b_("<h1>Hello, Jonathan!</h1>") in r.body
 
         r = app.get('/index.html?name=World')
         assert r.status_int == 200
-        assert "<h1>Hello, World!</h1>" in r.body
+        assert b_("<h1>Hello, World!</h1>") in r.body
 
         error_msg = None
         try:
             r = app.get('/badtemplate.html')
-        except Exception, e:
+        except Exception as e:
             for error_f in error_formatters:
                 error_msg = error_f(e)
                 if error_msg:
@@ -1387,7 +1422,7 @@ class TestEngines(PecanTestCase):
         app = TestApp(Pecan(RootController()))
         r = app.get('/')
         assert r.status_int == 200
-        result = dict(loads(r.body))
+        result = dict(loads(r.body.decode()))
         assert result == expected_result
 
     def test_override_template(self):
@@ -1400,7 +1435,7 @@ class TestEngines(PecanTestCase):
         app = TestApp(Pecan(RootController()))
         r = app.get('/')
         assert r.status_int == 200
-        assert 'Override' in r.body
+        assert b_('Override') in r.body
         assert r.content_type == 'text/plain'
 
     def test_render(self):
@@ -1414,4 +1449,4 @@ class TestEngines(PecanTestCase):
         )
         r = app.get('/')
         assert r.status_int == 200
-        assert "<h1>Hello, Jonathan!</h1>" in r.body
+        assert b_("<h1>Hello, Jonathan!</h1>") in r.body
